@@ -34,6 +34,7 @@ static char *rcsid = "$Id: ckconfig.c,v 1.9 1999/02/26 16:08:50 sob RELEASE sob
 #define  HOST_ACCESS  1
 #endif
 #include <stdio.h>
+#include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include "pathnames.h"
@@ -104,7 +105,5 @@ main()
     printf("of the server.\n");
   } else
     printf("ok.\n");
-    exit(0);
+  return(0);
 }
-
-
