@@ -31,9 +31,14 @@
 
 #include "config.h"
 
+#ifdef TSOL
+#include <tsol/priv.h>
+#endif
+
 #include <stdio.h>
 #include <errno.h>
 #include <string.h>
+#include <stdlib.h>
 #ifdef SYSSYSLOG
 #include <sys/syslog.h>
 #else
@@ -72,7 +77,12 @@ char *progname;
 /*************************************************************************/
 
 int
+#ifdef __STDC__
 parsetime(char *whattime)
+#else
+parsetime(whattime)
+char *whattime;
+#endif
 {
     static char *days[] =
     {"Su", "Mo", "Tu", "We", "Th", "Fr", "Sa", "Wk"};
@@ -98,24 +108,26 @@ parsetime(char *whattime)
             if (strncmp(days[loop], whattime, 2) == 0) {
                 whattime += 2;
                 match = 1;
-                if ((wday == loop) | ((loop == 7) && wday && (wday < 6)))
+                if ((wday == loop) || ((loop == 7) && wday && (wday < 6)))
                     validday = 1;
             }
         }
     }
 
-    if (strncmp(whattime, "Any", 3) == 0) {
-        validday = 1;
-        whattime += 3;
+    if (!validday) {
+        if (strncmp(whattime, "Any", 3) == 0) {
+            validday = 1;
+            whattime += 3;
+        }
+        else
+            return (0);
     }
-    if (!validday)
-        return (0);
 
     if (sscanf(whattime, "%d-%d", &start, &stop) == 2) {
         ltime = curtime->tm_min + 100 * curtime->tm_hour;
-        if ((start < stop) && ((ltime > start) && ltime < stop))
+        if ((start < stop) && ((ltime >= start) && ltime < stop))
             return (1);
-        if ((start > stop) && ((ltime > start) || ltime < stop))
+        if ((start > stop) && ((ltime >= start) || ltime < stop))
             return (1);
     } else
         return (1);
@@ -131,7 +143,12 @@ parsetime(char *whattime)
 /*************************************************************************/
 
 int
+#ifdef __STDC__
 validtime(char *ptr)
+#else
+validtime(ptr)
+char *ptr;
+#endif
 {
     char *nextptr;
     int good;
@@ -149,7 +166,14 @@ validtime(char *ptr)
     }
 }
 
+int
+#ifdef __STDC__
 acl_getlimit(char *aclbuf, char *class)
+#else
+acl_getlimit(aclbuf,class)
+char *aclbuf;
+char *class;
+#endif
 {
     char *crptr,
      *ptr,
@@ -158,26 +182,32 @@ acl_getlimit(char *aclbuf, char *class)
 
     while (*aclbuf != '\0') {
         if (strncasecmp(aclbuf, "limit", 5) == 0) {
-            for (crptr = aclbuf; *crptr++ != '\n' && *crptr;) ;
+            for (crptr = aclbuf; *crptr++ != '\n';) ;
             *--crptr = '\0';
             strcpy(linebuf, aclbuf);
             *crptr = '\n';
-	    printf(""); /* XXX - MvS - prevent coredump (??? Yes!) */
             (void) strtok(linebuf, " \t");  /* returns "limit" */
-            if (strcmp(class, strtok(NULL, " \t")) == 0) {
-                limit = atoi(strtok(NULL, " \t"));  /* returns limit <n> */
-                if ((ptr = strtok(NULL, " \t")) && validtime(ptr))
-                    return (limit);
+            if ((ptr = strtok(NULL, " \t")) && (strcmp(class, ptr) == 0)) {
+                if ((ptr = strtok(NULL, " \t"))) {
+                    limit = atoi(ptr);  /* returns limit <n> */
+                    if ((ptr = strtok(NULL, " \t")) && validtime(ptr))
+                        return (limit);
+                }
             }
         }
         while (*aclbuf && *aclbuf++ != '\n') ;
     }
 
-    return (0);
-
+    return (-1);
 }
 
+int
+#ifdef __STDC__
 acl_countusers(char *class)
+#else
+acl_countusers(class)
+char *class;
+#endif
 {
     int pidfd,
       count,
@@ -191,33 +221,80 @@ acl_countusers(char *class)
 struct flock arg;
 #endif
 
+#ifdef TSOL
+    int retval;
+#endif
+
     sprintf(pidfile, _PATH_PIDNAMES, class);
-    pidfd = open(pidfile, O_RDONLY, 0644);
+    pidfd = open(pidfile, O_RDWR, 0644);
     if (pidfd == -1) {
         return (0);
     }
-    lseek(pidfd, 0, L_SET);
+
+#ifdef HAVE_FLOCK
+    while (flock(pidfd, LOCK_EX)) {
+#ifndef NO_PID_SLEEP_MSGS
+        syslog(LOG_ERR, "sleeping: flock of pid file failed: %m");
+#endif
+#else
+    arg.l_type = F_WRLCK;
+    arg.l_whence = arg.l_start = arg.l_len = 0;
+#ifdef TSOL
+    if ((retval = set_effective_priv(PRIV_ON, 1, PRIV_FILE_LOCK)) != 0)
+    {
+        syslog(LOG_ERR, "Cannot add PRIV_FILE_LOCK to eff. priv. set");
+    }
+#endif
+    while ( -1 == fcntl( pidfd, F_SETLK, &arg) ) {
+#ifndef NO_PID_SLEEP_MSGS
+	syslog(LOG_ERR, "sleeping: fcntl lock of pid file failed: %m");
+#endif
+#endif /* HAVE_FLOCK */
+        sleep(1);
+    }
+#ifndef HAVE_FLOCK
+#ifdef TSOL
+    if ((retval = set_effective_priv(PRIV_OFF, 1, PRIV_FILE_LOCK)) != 0)
+    {
+        syslog(LOG_ERR, "Cannot remove PRIV_FILE_LOCK from eff. priv. set");
+    }
+#endif
+#endif /* HAVE_FLOCK */
 
     count = 0;
 
-    if (read(pidfd, buf, sizeof(buf)) == sizeof(buf)) {
+    if (read(pidfd, (void *)buf, sizeof(buf)) == sizeof(buf)) {
         for (which = 0; which < MAXUSERS; which++)
         if (buf[which]) {
             stat = kill(buf[which], SIGCONT);
             if (((stat == -1) && (errno == EPERM)) || !stat) {
                 if (strcmp(progname,"ftpcount")) {
 #if defined(SVR4)
-                    sprintf(line,"/bin/ps -l -p %d",buf[which]);
+#ifdef AIX
+                    sprintf(line,"/bin/ps %d",buf[which]);
+#elif defined(sun)
+                    sprintf(line,"/usr/ucb/ps auxww %d",buf[which]);
+#else
+#if defined (LINUX)
+                    sprintf(line,"/bin/ps axwww %d",buf[which]);
+#else
+                    sprintf(line,"/bin/ps -f -p %d",buf[which]);
+#endif
+#endif
 #elif defined(M_UNIX)
                     sprintf(line,"/bin/ps -f -p %d",buf[which]);
 #else
                     sprintf(line,"/bin/ps %d",buf[which]);
 #endif
                     ZeFile = popen(line, "r");
-                    rewind(ZeFile);
                     fgets(line, 1024, ZeFile);
                     fgets(line, 1024, ZeFile);
-                    printf("%s",line);
+{
+size_t i;
+for (i = strlen (line); (i > 0) && ((line[i-1] == ' ') || (line[i-1] == '\n')); --i)
+    line[i-1] = '\0';
+}
+		    printf("%s\n",line);
                     pclose(ZeFile);
                 }
                 count++;
@@ -228,23 +305,50 @@ struct flock arg;
     flock(pidfd, LOCK_UN);
 #else
     arg.l_type = F_UNLCK; arg.l_whence = arg.l_start = arg.l_len = 0;
-    fcntl(pidfd, F_SETLK, &arg);
+#ifdef TSOL
+    if ((retval = set_effective_priv(PRIV_ON, 1, PRIV_FILE_LOCK)) != 0)
+    {
+        syslog(LOG_ERR, "Cannot add PRIV_FILE_LOCK to eff. priv. set");
+    }
 #endif
+    fcntl(pidfd, F_SETLK, &arg);
+#ifdef TSOL
+    if ((retval = set_effective_priv(PRIV_OFF, 1, PRIV_FILE_LOCK)) != 0)
+    {
+        syslog(LOG_ERR, "Cannot remove PRIV_FILE_LOCK from eff. priv. set");
+    }
+#endif
+#endif /* HAVE_FLOCK */
     close(pidfd);
 
     return (count);
 }
 
 void
+#ifdef __STDC__
 new_list(struct c_list **list)
+#else
+new_list(list)
+struct c_list **list;
+#endif
 {
     (*list) = (struct c_list *) malloc(sizeof(struct c_list));
+if ( (*list) == NULL ) {
+  perror ("malloc error in new_list");
+  exit (0);
+}
 
     (*list)->next = NULL;
 }
 
 int
+#ifdef __STDC__
 add_list(char *class, struct c_list **list)
+#else
+add_list(class,list)
+char *class;
+struct c_list **list;
+#endif
 {
     struct c_list *cp;
 
@@ -254,15 +358,30 @@ add_list(char *class, struct c_list **list)
     }
 
     cp = (struct c_list *) malloc(sizeof(struct c_list));
+if (cp == NULL) {
+  perror ("malloc error in add_list");
+  exit (0);
+}
 
     cp->class = (char *) malloc(strlen(class) + 1);
+if (cp->class == NULL) {
+  perror ("malloc error in add_list");
+  exit (0);
+}
     strcpy(cp->class, class);
     cp->next = (*list)->next;
     (*list)->next = cp;
     return (1);
 }
 
+int
+#ifdef __STDC__
 main(int argc, char **argv)
+#else
+main(argc,argv)
+int argc;
+char **argv;
+#endif
 {
     FILE *accessfile;
     char class[80],
@@ -274,6 +393,17 @@ main(int argc, char **argv)
     struct stat finfo;
     struct c_list *list;
 
+#ifdef TSOL
+/* Before anything, clear the effective privilege set */
+    int retval;
+
+    if ((retval = set_effective_priv(PRIV_SET, 0)) != 0)
+    {
+        syslog(LOG_ERR, "ftp[count|who] cannot clear effective privileges!");
+        exit(1);
+    } 
+#endif
+
 	if (progname = strrchr(argv[0], '/'))  ++progname;
 	else  progname = argv[0];
 
@@ -295,12 +425,12 @@ main(int argc, char **argv)
         printf("%s: no service classes defined, no usage count kept\n",progname);
         exit(0);
     } else {
-        if (!(aclbuf = (char *) malloc(finfo.st_size + 1))) {
+        if (!(aclbuf = (char *) malloc((size_t)finfo.st_size + 1))) {
             perror("ftpcount: could not malloc aclbuf");
             exit(1);
         }
-        fread(aclbuf, finfo.st_size, 1, accessfile);
-        *(aclbuf + finfo.st_size) = '\0';
+        fread(aclbuf, (size_t)finfo.st_size, 1, accessfile);
+        *(aclbuf + (size_t)finfo.st_size) = '\0';
     }
 
     (void) new_list(&list);
@@ -319,16 +449,20 @@ main(int argc, char **argv)
                 ;
             } else {
                 limit = acl_getlimit(myaclbuf, class);
-            if (strcmp(progname,"ftpcount")) {
-                printf("Service class %s: \n", class);
-                printf("   - %3d users (%3d maximum)\n\n",
-            acl_countusers(class), limit);
-        }
-        else
-                printf("Service class %-20.20s - %3d users (%3d maximum)\n",
-                       class, acl_countusers(class), limit);
+                if (strcmp(progname,"ftpcount")) {
+                    printf("Service class %s: \n", class);
+                    printf("   - %3d users ", acl_countusers(class));
+                } else {
+                    printf("Service class %-20.20s - %3d users ",
+                            class, acl_countusers(class));
+                }
+                if (limit == -1)
+                    printf("(no maximum)\n");
+                else
+                    printf("(%3d maximum)\n", limit);
             }
         }
         while (*myaclbuf && *myaclbuf++ != '\n') ;
     }
+    return(0);
 }
