@@ -55,10 +55,15 @@ int denyoffset = 10;            /* default deny time   */
 int discoffset = 5;             /* default disc time   */
 char *message = "System shutdown at %s";    /* default message     */
 
-extern version[];
+extern char version[];
 
 void
+#ifdef __STDC__
 massage(char *buf)
+#else
+massage(buf)
+char *buf;
+#endif
 {
     char *sp = NULL;
     char *ptr;
@@ -102,7 +107,7 @@ massage(char *buf)
                 break;
             case '\0':
                 return;
-                break;
+                /* break; */
             default:
                 i = i + 1;
                 break;
@@ -119,9 +124,15 @@ massage(char *buf)
 }
 
 int
+#ifdef __STDC__
 main(int argc, char **argv)
+#else
+main(argc,argv)
+int argc;
+char **argv;
+#endif
 {
-    time_t c_time;
+    time_t c_time = 0;
     struct tm *tp;
 
     char buf[BUFSIZ];
@@ -177,7 +188,7 @@ main(int argc, char **argv)
         exit(0);
     } else {
         if (!(aclbuf = (char *) malloc(finfo.st_size + 1))) {
-            perror("ftpcount: could not malloc aclbuf");
+            perror("ftpshut: could not malloc aclbuf");
             exit(1);
         }
         fread(aclbuf, finfo.st_size, 1, accessfile);
@@ -232,7 +243,7 @@ main(int argc, char **argv)
                 argv[0]);
         fprintf(stderr, "       %s [-d min] [-l min] HHMM [\"message\"]\n",
                 argv[0]);
-        return(1);
+        exit(1);
     }
     /* do we have a shutdown message? */
 
@@ -245,12 +256,12 @@ main(int argc, char **argv)
 
     if ( sp == NULL ) {
         fprintf(stderr, "No shutdown file defined in ftpaccess file.\n");
-        return(1);
+        exit(1);
     }
     
     if ( (fp = fopen(sp, "w")) == NULL )  {
         perror("Couldn't open shutdown file");
-        return(1);
+        exit(1);
     }
 
     fprintf(fp, "%.4d %.2d %.2d %.2d %.2d %.4d %.4d\n",
@@ -263,4 +274,9 @@ main(int argc, char **argv)
             discoffset);
     fprintf(fp, "%s\n", buf);
     fclose(fp);
+    exit (0);
 }
+
+
+
+
