@@ -32,7 +32,7 @@
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)glob.c	5.9 (Berkeley) 2/25/91";
+static char sccsid[] = "@(#)$Id: glob.c,v 1.14 1999/02/26 16:08:50 sob RELEASE sob $ from glob.c 5.9 (Berkeley) 2/25/91";
 #endif /* not lint */
 
 /*
@@ -54,7 +54,6 @@ static char sccsid[] = "@(#)glob.c	5.9 (Berkeley) 2/25/91";
 #include <errno.h>
 #include <stdio.h>
 #include <string.h>
-#include <stdlib.h>
 
 #define	QUOTE 0200
 #define	TRIM 0177
@@ -64,9 +63,13 @@ static char sccsid[] = "@(#)glob.c	5.9 (Berkeley) 2/25/91";
 
 static	char **gargv;		/* Pointer to the (stack) arglist */
 static	int gargc;		/* Number args in gargv */
-static	int gnleft;
+static	size_t gnleft;
 static	short gflag;
-static	int tglob();
+#ifdef __STDC__
+static int tglob(register char);
+#else
+static int tglob();
+#endif
 char	**ftpglob();
 char	*globerr;
 char	*home;
@@ -75,7 +78,7 @@ char	*strspl();
 static	char *strend();
 char	**copyblk();
 
-static void acollect(), addpath(), collect(), expand(), Gcat();
+static void acollect(), collect(), expand(), Gcat();
 static void ginit(), matchdir(), rscan(), sort();
 static int amatch(), execbrc(), match();
 
@@ -88,15 +91,42 @@ static	int globbed;
 static	char *entp;
 static	char **sortbas;
 
+#ifdef __STDC__
+static void addpath(char); 
+void blkfree(char **);
+int letter(register char);
+int digit(register char);
+int gethdir(char *);
+int any(register int, register char *);
+#else
+static void addpath();
+void blkfree();
+int letter();
+int digit();
+int gethdir();
+int any();
+#endif
+
+extern void fatal(const char*);
+
 char **
+#ifdef __STDC__
+ftpglob(register char *v)
+#else
 ftpglob(v)
-	register char *v;
+register char *v;
+#endif
 {
 	char agpath[BUFSIZ];
 	char *agargv[GAVSIZ];
 	char *vv[2];
+	if (strstr(v, "../*/..") != (char *)NULL){
+		globerr = "Arguments too long";
+		return(0);
+	}
 	vv[0] = v;
-	vv[1] = 0;
+	vv[1] = NULL;
+	globerr = NULL;
 	gflag = 0;
 	rscan(vv, tglob);
 	if (gflag == 0) {
@@ -104,7 +134,7 @@ ftpglob(v)
 		return (copyblk(vv));
 	}
 
-	globerr = 0;
+	globerr = NULL;
 	gpath = agpath; gpathp = gpath; *gpathp = 0;
 	lastgpathp = &gpath[sizeof agpath - 2];
 	ginit(agargv); globcnt = 0;
@@ -117,8 +147,12 @@ ftpglob(v)
 }
 
 static void
+#ifdef __STDC__
+ginit(char ** agargv)
+#else
 ginit(agargv)
-	char **agargv;
+char ** agargv;
+#endif
 {
 
 	agargv[0] = 0; gargv = agargv; sortbas = agargv; gargc = 0;
@@ -126,8 +160,12 @@ ginit(agargv)
 }
 
 static void
+#ifdef __STDC__
+collect(register char *as)
+#else
 collect(as)
-	register char *as;
+register char *as;
+#endif
 {
 	if (eq(as, "{") || eq(as, "{}")) {
 		Gcat(as, "");
@@ -137,8 +175,12 @@ collect(as)
 }
 
 static void
+#ifdef __STDC__
+acollect(register char *as)
+#else
 acollect(as)
-	register char *as;
+register char *as;
+#endif
 {
 	register int ogargc = gargc;
 
@@ -166,8 +208,12 @@ sort()
 }
 
 static void
+#ifdef __STDC__
+expand(char *as)
+#else
 expand(as)
-	char *as;
+char *as;
+#endif
 {
 	register char *cs;
 	register char *sgpathp, *oldcs;
@@ -219,8 +265,12 @@ endit:
 }
 
 static void
+#ifdef __STDC__
+matchdir(char *pattern)
+#else
 matchdir(pattern)
-	char *pattern;
+char *pattern;
+#endif
 {
 	struct stat stb;
 
@@ -238,11 +288,11 @@ matchdir(pattern)
 			return;
 		goto patherr2;
 	}
-#ifdef __ELF__
-	if (fstat(dirfd(dirp), &stb) < 0)
-#else
-	if (fstat(dirp->dd_fd, &stb) < 0)
-#endif
+#ifdef HAVE_DIRFD
+       if (fstat(dirfd(dirp), &stb) < 0)
+#else /* HAVE_DIRFD */
+        if (fstat(dirp->dd_fd, &stb) < 0)
+#endif /* HAVE_DIRFD */
 		goto patherr1;
 	if (!isdir(stb)) {
 		errno = ENOTDIR;
@@ -266,8 +316,12 @@ patherr2:
 }
 
 static int
+#ifdef __STDC__
+execbrc(char *p, char *s)
+#else
 execbrc(p, s)
-	char *p, *s;
+char *p, *s;
+#endif
 {
 	char restbuf[BUFSIZ + 2];
 	register char *pe, *pm, *pl;
@@ -346,8 +400,12 @@ doit:
 }
 
 static int
-match(s, p)
-	char *s, *p;
+#ifdef __STDC__
+match(char *s, char *p)
+#else
+match(s,p)
+char *s, *p;
+#endif
 {
 	register int c;
 	register char *sentp;
@@ -364,8 +422,12 @@ match(s, p)
 }
 
 static int
+#ifdef __STDC__
+amatch(char *s, char *p)
+#else
 amatch(s, p)
-	register char *s, *p;
+char *s, *p;
+#endif
 {
 	register int scc;
 	int ok, lc;
@@ -453,9 +515,15 @@ slash:
 	}
 }
 
-static
-Gmatch(s, p)
-	register char *s, *p;
+/* This function appears to be unused, so why waste time and space on it? */
+#if 0 == 1
+static int
+#ifdef __STDC__
+Gmatch(register char *s, register char *p)
+#else
+Gmatch(s,p)
+register char *s, *p;
+#endif
 {
 	register int scc;
 	int ok, lc;
@@ -512,12 +580,17 @@ Gmatch(s, p)
 		}
 	}
 }
+#endif    /* Gmatch exclusion */
 
 static void
-Gcat(s1, s2)
-	register char *s1, *s2;
+#ifdef __STDC__
+Gcat(register char *s1, register char *s2)
+#else
+Gcat(s1,s2)
+register char *s1, *s2;
+#endif
 {
-	register int len = strlen(s1) + strlen(s2) + 1;
+	register size_t len = strlen(s1) + strlen(s2) + 1;
 
 	if (len >= gnleft || gargc >= GAVSIZ - 1)
 		globerr = "Arguments too long";
@@ -530,8 +603,12 @@ Gcat(s1, s2)
 }
 
 static void
+#ifdef __STDC__
+addpath(char c)
+#else
 addpath(c)
-	char c;
+char c;
+#endif
 {
 
 	if (gpathp >= lastgpathp)
@@ -543,71 +620,72 @@ addpath(c)
 }
 
 static void
-rscan(t, f)
-	register char **t;
-	int (*f)();
+#ifdef __STDC__
+rscan(register char **t, int (*f)(register char))
+#else
+rscan(t,f)
+register char **t;
+int (*f)();
+#endif
 {
 	register char *p, c;
 
 	while (p = *t++) {
-		if (f == tglob)
-			if (*p == '~')
-				gflag |= 2;
-			else if (eq(p, "{") || eq(p, "{}"))
-				continue;
+		if (*p == '~')
+			gflag |= 2;
+		else if (eq(p, "{") || eq(p, "{}"))
+			continue;
 		while (c = *p++)
 			(*f)(c);
 	}
 }
-/*
-static
-scan(t, f)
-	register char **t;
-	int (*f)();
-{
-	register char *p, c;
-
-	while (p = *t++)
-		while (c = *p)
-			*p++ = (*f)(c);
-} */
-
-static
+static int
+#ifdef __STDC__
+tglob(register char c)
+#else
 tglob(c)
-	register char c;
+register char c;
+#endif
 {
 
 	if (any(c, globchars))
 		gflag |= c == '{' ? 2 : 1;
 	return (c);
 }
-/*
-static
-trim(c)
-	char c;
-{
-
-	return (c & TRIM);
-} */
-
 
+int
+#ifdef __STDC__
+letter(register char c)
+#else
 letter(c)
-	register char c;
+register char c;
+#endif
 {
 
-	return (c >= 'a' && c <= 'z' || c >= 'A' && c <= 'Z' || c == '_');
+	return ((c >= 'a') && (c <= 'z') || (c >= 'A') && (c <= 'Z')
+		|| (c == '_'));
 }
 
+int
+#ifdef __STDC__
+digit(register char c)
+#else
 digit(c)
-	register char c;
+register char c;
+#endif
 {
 
 	return (c >= '0' && c <= '9');
 }
 
-any(c, s)
-	register int c;
-	register char *s;
+int
+#ifdef __STDC__
+any(register int c, register char *s)
+#else
+any(c,s)
+register int c;
+register char *s;
+#endif
 {
 
 	while (*s)
@@ -615,8 +693,14 @@ any(c, s)
 			return(1);
 	return(0);
 }
+
+int
+#ifdef __STDC__
+blklen(register char **av)
+#else
 blklen(av)
-	register char **av;
+register char **av;
+#endif
 {
 	register int i = 0;
 
@@ -626,9 +710,12 @@ blklen(av)
 }
 
 char **
+#ifdef __STDC__
+blkcpy(char **oav, register char **bv)
+#else
 blkcpy(oav, bv)
-	char **oav;
-	register char **bv;
+char **oav, **bv;
+#endif
 {
 	register char **av = oav;
 
@@ -637,8 +724,13 @@ blkcpy(oav, bv)
 	return (oav);
 }
 
+void
+#ifdef __STDC__
+blkfree(char **av0)
+#else
 blkfree(av0)
-	char **av0;
+char **av0;
+#endif
 {
 	register char **av = av0;
 
@@ -647,10 +739,15 @@ blkfree(av0)
 }
 
 char *
+#ifdef __STDC__
+strspl(register char *cp, register char *dp)
+#else
 strspl(cp, dp)
-	register char *cp, *dp;
+register char *cp, *dp;
+#endif
 {
-	register char *ep = malloc((unsigned)(strlen(cp) + strlen(dp) + 1));
+	register char *ep = 
+	  (char *)malloc((unsigned)(strlen(cp) + strlen(dp) + 1));
 
 	if (ep == (char *)0)
 		fatal("Out of memory");
@@ -660,8 +757,12 @@ strspl(cp, dp)
 }
 
 char **
+#ifdef __STDC__
+copyblk(register char **v)
+#else
 copyblk(v)
-	register char **v;
+register char **v;
+#endif
 {
 	register char **nv = (char **)malloc((unsigned)((blklen(v) + 1) *
 						sizeof(char **)));
@@ -673,8 +774,12 @@ copyblk(v)
 
 static
 char *
+#ifdef __STDC__
+strend(register char *cp)
+#else
 strend(cp)
-	register char *cp;
+register char *cp;
+#endif
 {
 
 	while (*cp)
@@ -687,13 +792,20 @@ strend(cp)
  * user whose home directory is sought is currently.
  * We write the home directory of the user back there.
  */
+int
+#ifdef __STDC__
+gethdir(char *home)
+#else
 gethdir(home)
-	char *home;
+char *home;
+#endif
 {
 	register struct passwd *pp = getpwnam(home);
-
+	register char *root = NULL;
 	if (!pp || home + strlen(pp->pw_dir) >= lastgpathp)
 		return (1);
-	(void) strcpy(home, pp->pw_dir);
+	root = strstr(pp->pw_dir, "/./");
+        (void) strcpy(home, root?(root+2):pp->pw_dir);
+
 	return (0);
 }
