@@ -15,11 +15,14 @@
 # IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 # WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 #
-#	@(#)newvers.sh	5.3 (Berkeley) 10/31/88
+#	@(#)$Id: newvers.sh,v 1.16 1998/07/06 09:13:57 sob BETA-18 sob $ based on
+#       newvers.sh	5.3 (Berkeley) 10/31/88
 #
 if [ ! -r edit ]; then echo 0 > edit; fi
 touch edit
 awk '	{	edit = $1 + 1; }\
-END	{	printf "char version[] = \"Version wu-2.3(%d) ", edit > "vers.c";\
+END	{	printf "char version[] = \"Version wu-2.5.0(%d) ", edit > "vers.c";\
+#LANG=
+#LC_TIME=
 		printf "%d\n", edit > "edit"; }' < edit
-echo `date`'";' >> vers.c
+echo `LC_TIME=C date`'";' >> vers.c
