@@ -29,12 +29,23 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 
+#ifndef lint
+static char rcsid[] = "@(#)$Id: private.c,v 1.14 1999/02/26 16:08:50 sob RELEASE sob $";
+#endif /* not lint */
+
 #ifndef NO_PRIVATE
 
 #include "config.h"
 
 #include <stdio.h>
 #include <errno.h>
+
+#ifdef __STDC__
+extern char *strsep(char **, const char *);
+#else
+extern char *strsep();
+#endif
+
 #include <string.h>
 #ifdef SYSSYSLOG
 #include <sys/syslog.h>
@@ -42,7 +53,6 @@
 #include <syslog.h>
 #endif
 #include <grp.h>
-#include <stdlib.h>
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -51,6 +61,18 @@
 #include "pathnames.h"
 #include "extensions.h"
 
+#ifdef SECUREOSF
+#define SecureWare /* Does this mean it works for all SecureWare? */
+#endif
+
+#ifdef HPUX_10_TRUSTED
+#include <hpsecurity.h>
+#endif
+
+#if defined(SecureWare) || defined(HPUX_10_TRUSTED)
+#include <prot.h>
+#endif
+
 #define MAXGROUPLEN 100
 char *passbuf = NULL;
 char groupname[MAXGROUPLEN];
@@ -70,28 +92,31 @@ extern int lgi_failure_threshold,
   autospout_free;
 extern char remotehost[],
   remoteaddr[],
+  remoteident[],
  *autospout;
 int group_attempts;
 
 void
+#ifdef __STDC__
 parsepriv(void)
+#else
+parsepriv()
+#endif
 {
     char *ptr;
     char *acptr = passbuf,
      *line;
-    char **argv[51],
-    **ap = (char **) argv,
+    char *argv[3],
      *p,
      *val;
-    struct acgrp *aptr;
+    struct acgrp *aptr,
+     *privtail = (struct acgrp *) NULL;
     struct group *gr;
+    int n;
 
     if (!passbuf || !(*passbuf))
         return;
 
-    /* build list, initialize to zero. */
-    privptr = (struct acgrp *) calloc(1, sizeof(struct acgrp));
-
     /* read through passbuf, stripping comments. */
     while (*acptr != '\0') {
         line = acptr;
@@ -106,34 +131,64 @@ parsepriv(void)
         if (*line == '\0')
             continue;
 
-        ap = (char **) argv;
-
         /* parse the lines... */
-        for (p = line; p != NULL;) {
-            while ((val = (char *) strsep(&p, ":\n")) != NULL
-                   && *val == '\0') ;
-            *ap = val;
-            if (**ap == ' ')
-                *ap = NULL;
-            *ap++;
+        for (n = 0, p = line; n < 3 && p != NULL; n++) {
+            val = (char *) strsep(&p, ":\n");
+            argv[n] = val;
+            if ((argv[n][0] == ' ') || (argv[n][0] == '\0'))
+                argv[n] = NULL;
         }
-        *ap = 0;
+        /* check their were 3 fields, if not skip the line... */
+        if (n != 3 || p != NULL)
+            continue;
 
-        if ((gr = getgrnam((char *) argv[2])) != NULL)  {
+if (argv[0] && argv[2]) {
+if (argv[2][0] == '%') {
+  gid_t gid = atoi (argv[2]+1);
+        if ((gr = getgrgid(gid)) != NULL)  {
             aptr = (struct acgrp *) calloc(1, sizeof(struct acgrp));
-            
-            /* if we didn't read 3 things, skip that line... */
+if (aptr == NULL) {
+  syslog (LOG_ERR, "calloc error in parsepriv");
+  exit (0);
+}
 
-            /* add element to beginning of list */
-
-            aptr->next = privptr;
-            privptr = aptr;
+            /* add element to end of list */
+            if (privtail)
+                privtail->next = aptr;
+            privtail = aptr;
+            if (!privptr)
+                privptr = aptr;
 
             strcpy(aptr->gname, (char *) argv[0]);
+if (argv[1] == NULL) aptr->gpass[0] = '\0'; else
+            strcpy(aptr->gpass, (char *) argv[1]);
+            strcpy(aptr->gr_name, gr->gr_name);
+            aptr->gr_gid = gid;
+        }
+} else {
+        if ((gr = getgrnam((char *) argv[2])) != NULL)  {
+            aptr = (struct acgrp *) calloc(1, sizeof(struct acgrp));
+if (aptr == NULL) {
+  syslog (LOG_ERR, "calloc error in parsepriv");
+  exit (0);
+}
+            
+            /* add element to end of list */
+            if (privtail)
+                privtail->next = aptr;
+            privtail = aptr;
+            if (!privptr)
+                privptr = aptr;
+
+            strcpy(aptr->gname, (char *) argv[0]);
+if (argv[1] == NULL) aptr->gpass[0] = '\0'; else
             strcpy(aptr->gpass, (char *) argv[1]);
             strcpy(aptr->gr_name, (char *) argv[2]);
             aptr->gr_gid = gr->gr_gid;
         }
+}
+	endgrent();
+}
     }
 }
 
@@ -144,40 +199,49 @@ parsepriv(void)
 /*************************************************************************/
 
 void
+#ifdef __STDC__
 priv_setup(char *path)
+#else
+priv_setup(path)
+char *path;
+#endif
 {
     FILE *prvfile;
     struct stat finfo;
 
     passbuf = (char *) NULL;
 
-    if (stat(path, &finfo) != 0) {
-        syslog(LOG_ERR, "cannot stat private access file %s: %s", path,
-               strerror(errno));
-        return;
-    }
     if ((prvfile = fopen(path, "r")) == NULL) {
         if (errno != ENOENT)
             syslog(LOG_ERR, "cannot open private access file %s: %s",
                    path, strerror(errno));
         return;
     }
+    if (fstat(fileno(prvfile), &finfo) != 0) {
+        syslog(LOG_ERR, "cannot fstat private access file %s: %s", path,
+               strerror(errno));
+        (void) fclose(prvfile);
+        return;
+    }
     if (finfo.st_size == 0) {
         passbuf = (char *) calloc(1, 1);
     } else {
-        if (!(passbuf = malloc((unsigned) finfo.st_size + 1))) {
+        if (!(passbuf = (char *)malloc((unsigned) finfo.st_size + 1))) {
             (void) syslog(LOG_ERR, "could not malloc passbuf (%d bytes)",
                           finfo.st_size + 1);
+            (void) fclose(prvfile);
             return;
         }
         if (!fread(passbuf, (size_t) finfo.st_size, 1, prvfile)) {
             (void) syslog(LOG_ERR, "error reading private access file %s: %s",
                           path, strerror(errno));
+            (void) fclose(prvfile);
+            return;
         }
         *(passbuf + finfo.st_size) = '\0';
     }
-
-	(void) parsepriv();
+    (void) fclose(prvfile);
+    (void) parsepriv();
 }
 
 /*************************************************************************/
@@ -188,12 +252,17 @@ priv_setup(char *path)
 /*************************************************************************/
 
 struct acgrp *
+#ifdef __STDC__
 priv_getent(char *group)
+#else
+priv_getent(group)
+char *group;
+#endif
 {
 	struct acgrp *ptr;
 
 	for (ptr = privptr; ptr; ptr=ptr->next)
-		if (!strcmp(group, ptr->gname))
+		if (!strcasecmp(group, ptr->gname))
 			return(ptr);
 
     return (NULL);
@@ -206,9 +275,14 @@ priv_getent(char *group)
 /*************************************************************************/
 
 void
+#ifdef __STDC__
 priv_group(char *group)
+#else
+priv_group(group)
+char *group;
+#endif
 {
-    if (strlen(group) < MAXGROUPLEN) {
+    if ((int)strlen(group) < MAXGROUPLEN) {
         strncpy(groupname, group, MAXGROUPLEN);
         group_given = 1;
         reply(200, "Request for access to group %s accepted.", group);
@@ -227,12 +301,24 @@ priv_group(char *group)
 /*************************************************************************/
 
 void
+#ifdef __STDC__
 priv_gpass(char *gpass)
+#else
+priv_gpass(gpass)
+char *gpass;
+#endif
 {
-    char *xgpass,
-     *salt;
+    char *xgpass = NULL;
 #ifndef NO_CRYPT_PROTO
+#ifdef __STDC__
     char *crypt(const char *, const char *);
+#else
+#ifdef _M_UNIX
+    char *crypt(char *, char *);
+#else
+    char *crypt();
+#endif
+#endif
 #endif
     struct acgrp *grp;
     struct group *gr;
@@ -247,26 +333,30 @@ priv_gpass(char *gpass)
      * give the group name again if they fail... */
     group_given = 0;
 
-    if (passbuf != NULL) {
-        grp = priv_getent(groupname);
+    grp = priv_getent(groupname);
+    if (passbuf && gpass && *gpass != '\0' && grp && *grp->gpass != '\0')
+#if defined(SecureWare) || defined(HPUX_10_TRUSTED)
+        xgpass = bigcrypt(gpass, grp->gpass);
+#else
+        xgpass = crypt(gpass, grp->gpass);
+#endif
 
-        if (grp == NULL)
-            salt = "xx";
-        else
-            salt = grp->gpass;
-
-        xgpass = crypt(gpass, salt);
-    } else
-        grp = NULL;
-
-    /* The strcmp does not catch null passwords! */
-    if (grp == NULL || *grp->gpass == '\0' || strcmp(xgpass, grp->gpass)) {
+    if (!(   (   (gpass       != NULL)
+              && (*gpass      != '\0')
+              && (grp         != NULL)
+              && (*grp->gpass != '\0')
+              && (strcmp (xgpass, grp->gpass) == 0))
+          || (   (   (gpass  == NULL)
+                  || (*gpass == '\0'))
+              && (grp         != NULL)
+              && (*grp->gpass == '\0'))
+       )) {
         reply(530, "Group access request incorrect.");
         grp = NULL;
         if (++group_attempts >= lgi_failure_threshold) {
             syslog(LOG_NOTICE,
-                   "repeated group access failures from %s [%s], group %s",
-                   remotehost, remoteaddr, groupname);
+                   "repeated group access failures from %s, group %s",
+                   remoteident, groupname);
             exit(0);
         }
         sleep(group_attempts);  /* slow down password crackers */
@@ -276,9 +366,11 @@ priv_gpass(char *gpass)
     uid = geteuid();
     gid = grp->gr_gid;
 
+    delay_signaling(); /* we can't allow any signals while euid==0: kinch */
     seteuid(0);
     setegid(gid);
     seteuid(uid);
+    enable_signaling(); /* we can allow signals once again: kinch */
 
     reply(200, "Group access enabled.");
     group_attempts = 0;
