@@ -55,7 +55,11 @@ static int g_fds,
   g_opts;
 static char *bp;
 
+#ifdef __STDC__
 int treewalk(char *path, int (*fn) ( /* ??? */ ), int maxfds, int opts);
+#else
+int treewalk();
+#endif
 
 /*
  * cycle through the directories at the top of the tree, otherwise, once
@@ -83,7 +87,11 @@ walk(register char *name)
 	struct stat sb;
 	FD cur;
 	char *save,
+#ifdef __STDC__
 	 *strcpy(char *, const char *);
+#else
+	 *strcpy();
+#endif
 
 	if (!freep)
 		freep = &cur;
@@ -283,12 +291,14 @@ chwalk(register char *name)
 	return (rval);
 }
 
+#ifndef HAVE_FTW
 /* S5 compatible ftw(BA_LIB) */
 int
 ftw(char *path, int (*fn) ( /* ??? */ ), int maxfds)
 {
 	return (treewalk(path, fn, maxfds, 0));
 }
+#endif
 
 int
 treewalk(char *path, int (*fn) ( /* ??? */ ), int maxfds, int opts)
