@@ -18,10 +18,12 @@
  */
 
 #if defined(LIBC_SCCS) && !defined(lint)
-static char sccsid[] = "@(#)getusershell.c	5.6 (Berkeley) 6/1/90";
+static char sccsid[] = "@(#)$Id: getusershell.c,v 1.8 1999/02/26 16:07:47 sob RELEASE sob $";
 
 #endif /* LIBC_SCCS and not lint */
 
+#ifndef _AIX
+
 #include "../src/config.h"
 #include <sys/types.h>
 #include <sys/param.h>
@@ -31,6 +33,7 @@ static char sccsid[] = "@(#)getusershell.c	5.6 (Berkeley) 6/1/90";
 #include <ctype.h>
 #include <stdio.h>
 
+
 #define SHELLS "/etc/shells"
 
 /*
@@ -48,7 +51,11 @@ static char **initshells();
  * Get a list of shells from SHELLS, if it exists.
  */
 char *
+#ifdef __STDC__
 getusershell(void)
+#else
+getusershell()
+#endif
 {
 	char *ret;
 
@@ -57,11 +64,15 @@ getusershell(void)
 	ret = *curshell;
 	if (ret != NULL)
 		curshell++;
-	return (ret);
+	return(ret);
 }
 
 void
+#ifdef __STDC__
 endusershell(void)
+#else
+endusershell()
+#endif
 {
 	if (shells != NULL)
 		free((char *) shells);
@@ -73,13 +84,21 @@ endusershell(void)
 }
 
 void
+#ifdef __STDC__
 setusershell(void)
+#else
+setusershell()
+#endif
 {
 	curshell = initshells();
 }
 
 static char **
+#ifdef __STDC__
 initshells(void)
+#else
+initshells()
+#endif
 {
 	register char **sp,
 	 *cp;
@@ -126,3 +145,72 @@ initshells(void)
 	(void) fclose(fp);
 	return (shells);
 }
+
+#else /* it is AIX */
+		 
+
+/* emulate getusershell for AIX */
+
+#include <userconf.h>
+#include <userpw.h>
+#include <sys/audit.h>
+#include <usersec.h>
+
+static int GETUSERSHELL_opened=0;
+static char **GETUSERSHELL_shells;
+static int GETUSERSHELL_current;
+			  
+
+char *getusershell()
+{		
+	static char *val;
+	static char *list;
+	static char *retVal;
+	int n;
+					
+	if (!GETUSERSHELL_opened)
+	{
+		if(getconfattr(SC_SYS_LOGIN,SC_SHELLS,(void *)&val,SEC_LIST))
+		{
+			return(NULL);
+		}
+		GETUSERSHELL_opened = 1;
+		GETUSERSHELL_current = 0;
+		list = val;
+	}	   
+				   
+	if ( (list != NULL) && (*list != NULL) )
+	{
+	while (list && *list)
+		list++;
+		
+	*list = '\0';		
+	
+	retVal = val;
+	list++; 
+	val = list;
+	}
+	else
+		retVal = NULL;
+	
+	return(retVal);
+	
+}
+
+void setusershell()
+{
+	GETUSERSHELL_opened = 0;
+}
+
+void endusershell()
+{
+	GETUSERSHELL_opened = 0;
+}
+
+	
+				   
+	
+	
+	
+	
+#endif
